package List;
# List module: linked lists using references, plus as_string limit
#
#	build the equivalent of the Haskell-style data decln:
#		data list = nil or cons( scalar head, list tail )
#	representing nil as [], and cons(h,t) as [h,t] (a 2-tuple)

use strict;
use warnings;
use Function::Parameters qw(:strict);
use Data::Dumper;

my $as_string_limit = 0;        # limit for as_string ("display first N").  0 means unlimited.


# $l = nil(): - return an empty list
#
fun nil()
{
	return [];
}

# $l = cons( $head, $tail ) - return a new list node.
#	$head becomes the head of the new list, and $tail the tail.
#
fun cons( $head, $tail )
{
	return [ $head, $tail ];
}

# $isnil = isnil( $list ) - return true iff the given list is nil
#
fun isnil( $list )
{
	die "List::isnil, bad list ".Dumper($list) unless
		ref($list) eq "ARRAY" && (@$list == 0 || @$list == 2);
	return @$list == 0 ? 1 : 0;
}

# ( $head, $tail ) = headtail( $list ) - break nonempty list into head and tail
#
fun headtail( $list )
{
	die "List::headtail, bad list ".Dumper($list) unless
		ref($list) eq "ARRAY" && (@$list == 0 || @$list == 2);
	die "List::headtail, empty list\n" if @$list == 0;
	my( $h, $t ) = @$list;
	return ( $h, $t );
}

# $len = len( $list ) - return the length of the given list
#
fun len( $list )
{
	my $len = 0;
	while( ! isnil($list) )
	{
		( my $h, $list ) = headtail($list);
		$len++;
	}
	return $len;
}

# $revlist = rev( $list ) - return the reverse of $list
#
fun rev( $list )
{
	my $result = nil();
	while( ! isnil($list) )
	{
		( my $h, $list ) = headtail($list);
		$result = cons( $h, $result );
	}
	return $result;
}

# set_as_string_limit( $n ) - set the as_string_limit limit to N
fun set_as_string_limit( $n )
{
	$as_string_limit = $n;
}

# $str = as_string( $list, [$limit] ) - return the printable form of the given list
#
fun as_string( $list, $limit = $as_string_limit )
{
	my $str = "";
	for( my $i = 1; ! isnil($list) && ($limit == 0 || $i <= $limit); $i++ )
	{
		( my $h, $list ) = headtail($list);
		$str .= "$h,";
	}
	chop $str;                          # remove trailing ','
	$str .= "..." unless isnil($list);  # must show that list has been cutoff!
	return "[ $str ]";
}

1;
